@@ -35,17 +35,23 @@ static gchar *contributors[] = {
 	"Christopher Blizzard",
 	N_("Jerome Bolliet"),
 	"Andreas Bolsch",
+	"Dave Bordoley",
 	"Dario Bressanini",
 	"Emmanuel Briot",
 	"Stephen Browne",
 	"Marcus Brubaker",
 	"Christian Bucher",
+	"Ross Burton",
+	"Brian Cameron",
 	"Dave Camp",
 	"Didier Carlier",
 	"Anders Carlsson",
 	"Chema Celorio",
 	"Damon Chaplin",
 	"Kevin Charter",
+	/* if your encoding allows it, use eacute (U00E9) for the first 
+	 * and the last 'e' in 'Chenede' */
+	N_("Erwann Chenede"),
 	/* CJK people please use (U5F35) (U570B) (U51A0) to replace
 	 * this name */
 	N_("Abel Cheung"),
@@ -55,9 +61,15 @@ static gchar *contributors[] = {
 	"Andrew Clausen",
 	"Jeremy Collins",
 	"Rusty Conover",
+	"Pat Costello",
 	"Alan Cox",
+	"Dennis Cranston",
 	"Mark Crichton",
+	/* if you encoding allows it, use eacute (U00E9) for the two
+	 * 'e' of 'Frederic' */
+	N_("Frederic Crozat"),
 	"Andreas Czechanowski",
+	"Johan Dahlin",
 	"Dan Damian",
 	"Phil Dawes",
 	"Fatih Demir",
@@ -67,12 +79,14 @@ static gchar *contributors[] = {
 	"Dietmar Maurer",
 	"Feico W. Dillema",
 	"Radek Doulik",
+	"Alex Duggan",
 	"Tom Dyas",
 	"Karl Eichwalder",
 	"John Ellis",
 	"Arturo Espinosa",
 	"Gus Estrella",
 	"David Etherton",
+	"Larry Ewing",
 	"Marc Ewing",
 	/* if your encoding allows it, use odoubleacute (U0151) for the 'o'
 	 * of 'Gergo' and use Eacute (U00C9) for the 'E' of 'Erdi' */
@@ -85,6 +99,8 @@ static gchar *contributors[] = {
 	"John Fleck",
 	N_("Raul Perusquia Flores"),
 	"Lawrence Foard",
+	"Glynn Foster",
+	"Ben FrantzDale",
 	"Jeff Freedman",
 	"David Abilleira Freijeiro",
 	"Nat Friedman",
@@ -112,9 +128,11 @@ static gchar *contributors[] = {
 	"Lars Hamann",
 	"Michael Hanni",
 	"Raja R Harinath",
+	"Heath Harrelson",
 	"Peter Hawkins",
 	"Scott Heavner",
 	"James Henstridge",
+        "Fernando Herrera",
 	"Richard Hestilow",
 	"Iain Holmes",
 	"David Huggins-Daines",
@@ -167,6 +185,7 @@ static gchar *contributors[] = {
 	"Paolo Maggi",
 	"Sam Magnuson",
 	"Ian Main",
+        "Jordi Mallach",
 	"Mandrake",
 	"Daniel Manrique",
 	"Kjartan Maraas",
@@ -190,6 +209,7 @@ static gchar *contributors[] = {
 	"Jaka Mocnik",
 	"Paolo Molaro",
 	"David Mosberger",
+	"Rodrigo Moya",
 	"Dan Mueth",
 	"Thomas Muldowney",
 	/* If your encodings allows it, use ntilde (U00F1) for the 'n'
@@ -199,6 +219,7 @@ static gchar *contributors[] = {
 	N_("Sung-Hyun Nam"),
 	"Deepa Natarajan",
 	"Karl Nelson",
+	"Elijah Newren",
 	"Asger Alstrup Nielsen",
 	"Seth Nickell",
 	"Eric Nielson",
@@ -229,6 +250,9 @@ static gchar *contributors[] = {
 	"Joe Pfeiffer",
 	"Ben Pierce",
 	"Chris Pinkham",
+	/* If your encoding allows it, use aacute (U00E1) for the 'a'
+	 * of 'German' and use ntilde (U00F1) for the 'n' of 'Caamano' */
+	N_("German Poo-Caamano"),
 	"Dick Porter",
 	"Tero Pulkkinen",
 	"The Rasterman",
@@ -239,17 +263,21 @@ static gchar *contributors[] = {
 	"Jens Christian Restemeier",
 	"Patrick Reynolds",
 	"Robert Richardson",
+	"Kristian Rietveld",
 	"Alex Roberts",
 	"Michel Roelofs",
 	"Ueli Rutishauser",
 	"Lars Rydlinge",
 	"Peter Ryland",
 	"Bibek Sahu",
+	"Arvind Samptur",
+	"S\xc3\xb8ren Sandmann",
 	"Pablo Saratxaga",
 	/* Name in Japanese is: U7b39 U5c71 U548c U5b8f */
 	N_("Kazuhiro Sasayama"),
 	"Carsten Schaar",
 	"Christian Schaller",
+	"Wayne Schuller",
 	"Franck Schneider",
 	"Ingo Schneider",
 	"Bernd Schroeder",
@@ -269,15 +297,18 @@ static gchar *contributors[] = {
 	"John Slee",
 	"Garrett Smith",
 	"Suzanna Smith",
+	"Andrew Sobala",
 	/* not really a person name :) translate its meaning */
 	N_("The Squeaky Rubber Gnome"),
 	"Maciej Stachowiak",
 	"Stalyn",
+	"Jeffrey Stedfast",
 	"Jakub Steiner",
 	"Ben Stern",
 	"Micah Stetson",
 	"Nathan Carl Summers",
 	N_("Istvan Szekeres"),
+	"HideToshi Tajima",
 	"Tristan Tarrant",
 	"Anthony Taylor",
 	"Owen Taylor",
@@ -288,18 +319,23 @@ static gchar *contributors[] = {
 	"Christoph Toshok",
 	"Tom Tromey",
 	"Jon Trowbridge",
+	"Vincent Untz",
 	N_("Manish Vachharajani"),
 	N_("Neil Vachharajani"),
+	"Thomas Vander Stichele",
 	"Daniel Veillard",
 	"Vendu",
 	"Andrew Veliath",
+	"Luis Villa",
 	"Marius Vollmer",
 	"Shawn Wagner",
-	"Matthias Warkus",
 	N_("Wanda the GNOME Fish"),
+	"Matthias Warkus",
+	"Jeff Waugh",
 	"Bruno Widmann",
 	"Robert Wilhelm",
 	"Sebastian Wilhelmi",
+	"James Willcox",
 	"David Winkler",
 	"Jeremy Wise",
 	"Roger Wolff",
