@@ -52,6 +52,7 @@
 #include "egg-spawn.h"
 
 #include "gnome-desktop-item.h"
+#include "gnome-desktop-i18n.h"
 
 #ifdef HAVE_STARTUP_NOTIFICATION
 #define SN_API_NOT_YET_FROZEN
@@ -598,22 +599,6 @@ gnome_desktop_item_new_from_uri (const char *uri,
 		return NULL;
 	}	    	
 	
-	if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_TYPE &&
-	    info->type == GNOME_VFS_FILE_TYPE_REGULAR &&
-	    info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE &&
-	    strcmp (info->mime_type, "application/x-gnome-app-info") != 0) {
-		g_set_error (error,
-			     /* FIXME: better errors */
-			     GNOME_DESKTOP_ITEM_ERROR,
-			     GNOME_DESKTOP_ITEM_ERROR_INVALID_TYPE,
-			     _("File '%s' has invalid MIME type: %s"),
-			     uri, info->mime_type);
-
-		gnome_vfs_file_info_unref (info);
-
-		return NULL;
-	}		
-
 	if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MTIME)
 		mtime = info->mtime;
 	else
@@ -1495,14 +1480,156 @@ sn_error_trap_pop (SnDisplay *display,
 	gdk_error_trap_pop ();
 }
 
-static void
-sn_child_setup_func (void *data)
-{
-	SnLauncherContext *sn_context = data;
+extern char **environ;
 
-	sn_launcher_context_setup_child_process (sn_context);
+static char **
+make_spawn_environment_for_sn_context (SnLauncherContext *sn_context,
+				       char             **envp)
+{
+	char **retval = NULL;
+	int    i, j;
+	int    desktop_startup_id_len;
+
+	if (envp == NULL)
+		envp = environ;
+	
+	for (i = 0; envp[i]; i++)
+		;
+
+	retval = g_new (char *, i + 2);
+
+	desktop_startup_id_len = strlen ("DESKTOP_STARTUP_ID");
+	
+	for (i = 0, j = 0; envp[i]; i++) {
+		if (strncmp (envp[i], "DESKTOP_STARTUP_ID", desktop_startup_id_len) != 0)
+		{
+			retval[j] = g_strdup (envp[i]);
+			++j;
+	        }
+	}
+
+	retval[j] = g_strdup_printf ("DESKTOP_STARTUP_ID=%s",
+				     sn_launcher_context_get_startup_id (sn_context));
+	++j;
+	retval[j] = NULL;
+
+	return retval;
 }
 
+/* This should be fairly long, as it's confusing to users if a startup
+ * ends when it shouldn't (it appears that the startup failed, and
+ * they have to relaunch the app). Also the timeout only matters when
+ * there are bugs and apps don't end their own startup sequence.
+ *
+ * This timeout is a "last resort" timeout that ignores whether the
+ * startup sequence has shown activity or not.  Metacity and the
+ * tasklist have smarter, and correspondingly able-to-be-shorter
+ * timeouts. The reason our timeout is dumb is that we don't monitor
+ * the sequence (don't use an SnMonitorContext)
+ */
+#define STARTUP_TIMEOUT_LENGTH (30 /* seconds */ * 1000)
+
+typedef struct
+{
+	GdkScreen *screen;
+	GSList *contexts;
+	guint timeout_id;
+} StartupTimeoutData;
+
+static void
+free_startup_timeout (void *data)
+{
+	StartupTimeoutData *std = data;
+
+	g_slist_foreach (std->contexts,
+			 (GFunc) sn_launcher_context_unref,
+			 NULL);
+	g_slist_free (std->contexts);
+
+	if (std->timeout_id != 0) {
+		g_source_remove (std->timeout_id);
+		std->timeout_id = 0;
+	}
+
+	g_free (std);
+}
+
+static gboolean
+startup_timeout (void *data)
+{
+	StartupTimeoutData *std = data;
+	GSList *tmp;
+	GTimeVal now;
+	int min_timeout;
+
+	min_timeout = STARTUP_TIMEOUT_LENGTH;
+	
+	g_get_current_time (&now);
+	
+	tmp = std->contexts;
+	while (tmp != NULL) {
+		SnLauncherContext *sn_context = tmp->data;
+		GSList *next = tmp->next;
+		long tv_sec, tv_usec;
+		double elapsed;
+		
+		sn_launcher_context_get_last_active_time (sn_context,
+							  &tv_sec, &tv_usec);
+
+		elapsed =
+			((((double)now.tv_sec - tv_sec) * G_USEC_PER_SEC +
+			  (now.tv_usec - tv_usec))) / 1000.0;
+
+		if (elapsed >= STARTUP_TIMEOUT_LENGTH) {
+			std->contexts = g_slist_remove (std->contexts,
+							sn_context);
+			sn_launcher_context_complete (sn_context);
+			sn_launcher_context_unref (sn_context);
+		} else {
+			min_timeout = MIN (min_timeout, (STARTUP_TIMEOUT_LENGTH - elapsed));
+		}
+		
+		tmp = next;
+	}
+
+	if (std->contexts == NULL) {
+		std->timeout_id = 0;
+	} else {
+		std->timeout_id = g_timeout_add (min_timeout,
+						 startup_timeout,
+						 std);
+	}
+
+	/* always remove this one, but we may have reinstalled another one. */
+	return FALSE;
+}
+
+static void
+add_startup_timeout (GdkScreen         *screen,
+		     SnLauncherContext *sn_context)
+{
+	StartupTimeoutData *data;
+
+	data = g_object_get_data (G_OBJECT (screen), "gnome-startup-data");
+	if (data == NULL) {
+		data = g_new (StartupTimeoutData, 1);
+		data->screen = screen;
+		data->contexts = NULL;
+		data->timeout_id = 0;
+		
+		g_object_set_data_full (G_OBJECT (screen), "gnome-startup-data",
+					data, free_startup_timeout);		
+	}
+
+	sn_launcher_context_ref (sn_context);
+	data->contexts = g_slist_prepend (data->contexts, sn_context);
+	
+	if (data->timeout_id == 0) {
+		data->timeout_id = g_timeout_add (STARTUP_TIMEOUT_LENGTH,
+						  startup_timeout,
+						  data);		
+	}
+}
 #endif /* HAVE_STARTUP_NOTIFICATION */
 
 static int
@@ -1518,6 +1645,7 @@ ditem_execute (const GnomeDesktopItem *item,
 	       gboolean append_paths,
 	       GError **error)
 {
+	char **free_me = NULL;
 	char **real_argv;
 	int i, ret;
 	char **term_argv = NULL;
@@ -1533,6 +1661,7 @@ ditem_execute (const GnomeDesktopItem *item,
 #ifdef HAVE_STARTUP_NOTIFICATION
 	SnLauncherContext *sn_context;
 	SnDisplay *sn_display;
+	const char *startup_class;
 #endif
 	
 	g_return_val_if_fail (item, -1);
@@ -1569,19 +1698,58 @@ ditem_execute (const GnomeDesktopItem *item,
 	 * to initiate, but why bother)
 	 */
 
-	if (gnome_desktop_item_get_boolean (item, "StartupNotify")) {
-		sn_context = sn_launcher_context_new (
-					sn_display,
-					screen ? gdk_screen_get_number (screen) :
-					         DefaultScreen (gdk_display));
+	startup_class = gnome_desktop_item_get_string (item,
+						       "StartupWMClass");
+	if (startup_class ||
+	    gnome_desktop_item_get_boolean (item, "StartupNotify")) {
+		const char *name;
+		const char *icon;
+
+		sn_context = sn_launcher_context_new (sn_display,
+						      screen ? gdk_screen_get_number (screen) :
+						      DefaultScreen (gdk_display));
+		
+		name = gnome_desktop_item_get_localestring (item,
+							    GNOME_DESKTOP_ITEM_NAME);
+
+		if (name == NULL)
+			name = gnome_desktop_item_get_localestring (item,
+								    GNOME_DESKTOP_ITEM_GENERIC_NAME);
+		
+		if (name != NULL) {
+			char *description;
+			
+			sn_launcher_context_set_name (sn_context, name);
+			
+			description = g_strdup_printf (_("Starting %s"), name);
+			
+			sn_launcher_context_set_description (sn_context, description);
+			
+			g_free (description);
+		}
+		
+		icon = gnome_desktop_item_get_string (item,
+						      GNOME_DESKTOP_ITEM_ICON);
+		
+		if (icon != NULL)
+			sn_launcher_context_set_icon_name (sn_context, icon);
+		
+		sn_launcher_context_set_workspace (sn_context, workspace);
+
+		if (startup_class != NULL)
+			sn_launcher_context_set_wmclass (sn_context,
+							 startup_class);
 	} else {
 		sn_context = NULL;
 	}
 #endif
 
-	if (screen)
-		envp = egg_make_spawn_environment_for_screen (
-				screen, envp);
+	if (screen) {
+		envp = egg_make_spawn_environment_for_screen (screen, envp);
+		if (free_me)
+			g_strfreev (free_me);
+		free_me = envp;
+	}
 	
 	do {
 		added_status = ADDED_NONE;
@@ -1637,17 +1805,39 @@ ditem_execute (const GnomeDesktopItem *item,
 		g_slist_foreach (vector_list, (GFunc)g_free, NULL);
 		g_slist_free (vector_list);
 
+#ifdef HAVE_STARTUP_NOTIFICATION
+		if (sn_context != NULL &&
+		    !sn_launcher_context_get_initiated (sn_context)) {
+
+			/* This means that we always use the first real_argv[0]
+			 * we select for the "binary name", but it's probably
+			 * OK to do that. Binary name isn't super-important
+			 * anyway, and we can't initiate twice, and we
+			 * must initiate prior to fork/exec.
+			 */
+			
+			sn_launcher_context_set_binary_name (sn_context,
+							     real_argv[0]);
+			
+			sn_launcher_context_initiate (sn_context,
+						      g_get_prgname () ? g_get_prgname () : "unknown",
+						      real_argv[0],
+						      CurrentTime);
+
+			envp = make_spawn_environment_for_sn_context (sn_context, envp);
+			if (free_me)
+				g_strfreev (free_me);
+			free_me = envp;
+		}
+#endif
+		
+		
 		if ( ! g_spawn_async (working_dir,
 				      real_argv,
 				      envp,
 				      G_SPAWN_SEARCH_PATH /* flags */,
-#ifdef HAVE_STARTUP_NOTIFICATION
-				      sn_context ? sn_child_setup_func : NULL,
-				      sn_context,
-#else
-				      NULL /* child_setup */,
-				      NULL /* user_data */,
-#endif
+				      NULL, /* child_setup_func */
+				      NULL, /* child_setup_func_data */
 				      &ret /* child_pid */,
 				      error)) {
 			/* The error was set for us,
@@ -1668,53 +1858,25 @@ ditem_execute (const GnomeDesktopItem *item,
 
 #ifdef HAVE_STARTUP_NOTIFICATION
 	if (sn_context != NULL) {
-		const char *name;
-		const char *icon;
-		
-		name = gnome_desktop_item_get_string (item,
-						      GNOME_DESKTOP_ITEM_NAME);
-		
-		if (name != NULL) {
-			char *description;
-
-			sn_launcher_context_set_name (sn_context, name);
-			
-			description = g_strdup_printf (_("Starting %s"), name);
-
-			sn_launcher_context_set_description (sn_context, description);
-			
-			g_free (description);
-		}
-		
-		icon = gnome_desktop_item_get_string (item,
-						      GNOME_DESKTOP_ITEM_ICON);
-		
-		if (icon != NULL)
-			sn_launcher_context_set_icon_name (sn_context, icon);
-		
-		sn_launcher_context_set_binary_name (sn_context,
-						     exec);
-
-		sn_launcher_context_set_workspace (sn_context, workspace);
-
-		sn_launcher_context_initiate (sn_context,
-					      g_get_prgname () ? g_get_prgname () : "unknown",
-					      exec,
-					      CurrentTime);
-		
+		if (ret < 0)
+			sn_launcher_context_complete (sn_context); /* end sequence */
+		else
+			add_startup_timeout (screen ? screen :
+					     gdk_display_get_default_screen (gdk_display_get_default ()),
+					     sn_context);
 		sn_launcher_context_unref (sn_context);
 	}
 	
 	sn_display_unref (sn_display);
-#endif
+#endif /* HAVE_STARTUP_NOTIFICATION */
 	
 	free_args (args);
 	
 	if (term_argv)
 		g_strfreev (term_argv);
 
-	if (screen)
-		g_free (envp);
+	if (free_me)
+		g_strfreev (free_me);
 
 	return ret;
 }
@@ -1953,6 +2115,48 @@ gnome_desktop_item_drop_uri_list (const GnomeDesktopItem *item,
 	return ret;
 }
 
+/**
+* gnome_desktop_item_drop_uri_list_with_env:
+* @item: A desktop item
+* @uri_list: text as gotten from a text/uri-list
+* @flags: FIXME
+* @envp: child's environment
+* @error: FIXME
+*
+* See gnome_desktop_item_drop_uri_list for a full description. This function
+* additionally passes an environment vector for the child process
+* which is to be launched.
+*
+* Returns: The value returned by #gnome_execute_async() upon execution of
+* the specified item or -1 on error.  If multiple instances are run, the
+* return of the last one is returned.
+*/
+int
+gnome_desktop_item_drop_uri_list_with_env (const GnomeDesktopItem *item,
+					   const char *uri_list,
+					   GnomeDesktopItemLaunchFlags flags,
+					   char                        **envp,
+					   GError **error)
+{
+	GList *li;
+	int ret;
+	GList *list = gnome_vfs_uri_list_parse (uri_list);
+
+	for (li = list; li != NULL; li = li->next) {
+		GnomeVFSURI *uri = li->data;
+		li->data = gnome_vfs_uri_to_string (uri, 0 /* hide_options */);
+		gnome_vfs_uri_unref (uri);
+	}
+
+	ret =  gnome_desktop_item_launch_with_env (
+			item, list, flags, envp, error);
+
+	g_list_foreach (list, (GFunc)g_free, NULL);
+	g_list_free (list);
+
+	return ret;
+}
+
 static gboolean
 exec_exists (const char *exec)
 {
